<?php

namespace Drupal\json_users_import\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\user\Entity\User;

/*
 * Contoller for Json User Import
 */

class JsonUsersImportController extends ControllerBase {

  /**
   * Returns the fields machine name from the user entity.
   *
   * @return array
   *   User fields machine name.
   */
  public function getUserFields() {
    $userAccountFields = array_filter( \Drupal::service('entity_field.manager')->getFieldDefinitions('user', 'user'), function ($fieldDefinition) {
        return $fieldDefinition instanceof \Drupal\field\FieldConfigInterface;
      }
    );
    return $userAccountFields;
  }

  /**
   * Returns user data if email id is exists.
   *
   * @param string|$email
   *   Email id from the currently uploaded json data.
   *
   * @return array
   *   User Id and Error message.
   */
  public function isEmailExist($email) {
    $userId = \Drupal::entityQuery('user')->condition('mail', $email)->execute();
    $userId = array_shift($userId);
    if($userId > 0) {
      return array ('id' => $userId, 'returnMsg' => $email.' is already exists!');
    }
  }

  /**
   * Returns user data if username is exists.
   *
   * @param string|$username
   *   Username from the currently uploaded json data.
   *
   * @return array
   *   User Id and Error message.
   */
  public function isUserNameExist($username) {
    $userId = \Drupal::entityQuery('user')->condition('name', $username)->range(0, 1)
        ->execute();
    $userId = array_shift($userId);
    if($userId > 0) {
      return array ('id' => $userId, 'returnMsg' => $username.' is already exists!');
    }
  }

  /**
   * Returns array if username is valid.
   *
   * @param string|$username
   *   Username from the currently uploaded json data.
   *
   * @return array
   *   Return FALSE, if username is valid and Error message.
   *   Otherwise return TRUE.
   */
  public function isUserNameValid($username) {
    $valid = TRUE;
    $returnMsg = '';
    if (substr($username, 0, 1) == ' ') {
      $valid = FALSE;
      $returnMsg = $username .' cannot begin with a space.';
    } else if (substr($username, -1) == ' ') {
      $valid = FALSE;
      $returnMsg = $username .' cannot end with a space.';
    } else if (strpos($username, '  ') !== FALSE) {
      $valid = FALSE;
      $returnMsg = $username .' cannot contain multiple spaces.';
    } else if (preg_match('/[^\x{80}-\x{F7} a-z0-9@+_.\'-]/i', $username)) {
      $valid = FALSE;
      $returnMsg = $username .' contains an illegal character.';
    } else if (mb_strlen($username) > 60 ) {
      $valid = FALSE;
      $returnMsg = $username .' is too long, it must be 60 characters or less.';
    }
    return array ('valid' => $valid, 'returnMsg' => $returnMsg);
  }

  /**
   * Create user data if user not exists.
   *
   * @param array|$value
   *   An array of user data from the currently uploaded json data.
   *
   * @param string|$emailKey
   *   Email key from the currently uploaded json data.
   *
   * @param string|$usernameKey
   *   Username key from the currently uploaded json data.
   *
   * @param array|$jsonKey
   *   An array of json key from the user import configuration.
   *
   * @return string
   *   Status of the created user record.
   */
  public function createUser($value, $emailKey, $usernameKey, $fields) {
    $config = \Drupal::config('json_users_import.import_configuration');
    $language = \Drupal::languageManager()->getCurrentLanguage()->getId();
    $statusMsg = '';
    $random = new \Drupal\Component\Utility\Random();
    $password = $random->name(7);
    // Creating the user account
    $user = User::create();
    $user->setEmail($value[$emailKey]);
    $user->setUsername($value[$usernameKey]);
    $user->setPassword($password);
    $user->enforceIsNew();
    $user->set('init', $value[$emailKey]);
    $user->set('langcode', $language);
    $user->set('preferred_langcode', $language);
    $user->set('preferred_admin_langcode', $language);
    $user->activate();
    $ignoreFields = array("email", "name", "password");
    foreach($fields as $fieldName => $fieldValue) {
      if (!in_array($fieldName, $ignoreFields)) {
        $user->set($fieldName,$value[$fieldValue]);
      }
    }
    // Save user account.
    $result = $user->save();
    if ($user->isActive()) {
      $userId = $user->id();
      // Send mail if the mail configuration is enabled
      if ($config->get('mail_login_options')) {
        $this->send_user_account_details($value[$emailKey], $userId,$password);
      }
      $statusMsg = 'User '. $value[$usernameKey] .' is created successfully!';
      \Drupal::logger('json_users_import')->notice($statusMsg);
    } else {
      $statusMsg = 'User '. $value[$usernameKey] .' is not created!';
      \Drupal::logger('json_users_import')->error($statusMsg);
    }
    return array ('statusMsg' => $statusMsg, 'userId' => $userId);
  }

  /**
   * Send the user credentials or one time password link.
   *
   * @param string|$email
   *   An array of user data from the currently uploaded json data.
   *
   * @param string|$userId
   *   Email key from the currently uploaded json data.
   *
   * @param string|$password
   *   Username key from the currently uploaded json data.
   *
   */
  function send_user_account_details($email, $userId, $password) {
    $config = \Drupal::config('json_users_import.import_configuration');
    $mailManager = \Drupal::service('plugin.manager.mail');
    $module = 'json_users_import';
    $key = 'user_created';
    $to = $email;
    if($config->get('mail_login_options') == 'send_onetime_login') {
      $params['message'] = $config->get('onetime_login_content');
      $params['password'] = '';
    } else {
      $params['message'] = $config->get('password_content');
      $params['password'] = $password;
    }
    $params['title'] = 'New account creation';
    $params['userId'] = $userId;
    $langcode = \Drupal::currentUser()->getPreferredLangcode();
    $send = true;
    $result = $mailManager->mail($module, $key, $to, $langcode, $params, NULL, $send);
    if ($result['result'] != true) {
      $message = t('There was a problem sending your email notification to @email.', array('@email' => $to));
      \Drupal::messenger()->addError($message);
      \Drupal::logger('mail-log')->error($message);
      return;
    }
    $message = t('An email notification has been sent to @email ', array('@email' => $to));
    \Drupal::messenger()->addStatus($message);
    \Drupal::logger('mail-log')->notice($message);
  }
} 
